@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009 Jakub Wilk <ubanus@users.sf.net>
+# Copyright © 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,12 +13,46 @@
 
 '''Bridge to the Gamera framework'''
 
-from gamera.core import load_image, init_gamera as _init_gamera
-from gamera.core import Image, RGB, ONEBIT, Dim, RGBPixel
-from gamera.plugins.pil_io import from_pil
+import os
+import re
+import sys
+
+from gamera.core import load_image as _load_image
+from gamera.core import init_gamera as init
+from gamera.core import Image, RGB, GREYSCALE, ONEBIT, Dim, RGBPixel
+from gamera.plugins.pil_io import from_pil as _from_pil
+
+try:
+    import Image as PIL
+except ImportError:
+    load_image = _load_image
+else:
+    def load_image(filename):
+        pil_image = PIL.open(filename)
+        if pil_image.mode == '1':
+            # Gamera supports importing only 8-bit and RGB from PIL:
+            pil_image = pil_image.convert('L')
+        # TODO: Try to load image without PIL, even if it is imported.
+        return _from_pil(pil_image)
+
+def colorspace_wrapper(plugin):
+
+    pixel_types = plugin.self_type.pixel_types
+
+    def new_plugin(image, method=[None]):
+        if image.data.pixel_type not in pixel_types:
+            if RGB in pixel_types:
+                image = image.to_rgb()
+            elif GREYSCALE in pixel_types:
+                image = image.to_greyscale()
+        if method[0] is None:
+            method[0] = plugin()
+        return method[0](image)
+
+    new_plugin.__name__ = plugin.__name__
+    return new_plugin
 
 def _load_methods():
-    import re
     replace_suffix = re.compile('_threshold$').sub
     class _methods:
         from gamera.plugins.threshold import abutaleb_threshold
@@ -35,7 +69,7 @@ def _load_methods():
         if name.startswith('_'):
             continue
         name = replace_suffix('', name)
-        methods[name] = plugin
+        methods[name] = colorspace_wrapper(plugin)
     return methods
 
 methods = _load_methods()
@@ -43,12 +77,4 @@ methods = _load_methods()
 def to_pil_1bpp(image):
     return image.to_greyscale().to_pil()
 
-def init():
-    import os
-    import sys
-    _old_stdout = sys.stdout
-    sys.stdout = file(os.devnull, 'w')
-    _init_gamera()
-    sys.stdout = _old_stdout
-
 # vim:ts=4 sw=4 et
